import time


class Timer(object):
    """Timer class measures time needed to execute operations in a given context.
    Should measured time be needed for further use, one can use time module.

    - Example::

          >>> from lhcsmapi.Timer import Timer
          >>> with Timer(): # doctest: +SKIP
          ...    for i in range(1000000):
          ...      i = i+1
          Elapsed: 0.17786049842834473 s.

    - Example::
          >>> import time
          >>> t = time.time()
          >>> for i in range(1000000):
          ...   i = i+1
          >>> time.time() - t # doctest: +SKIP
          0.1149590015411377
    """

    def __enter__(self):
        self.tstart = time.time()

    def __exit__(self, type, value, traceback):
        print('Elapsed: {:.3f} s.'.format(time.time() - self.tstart))
