import math
import pandas as pd
from copy import deepcopy
import matplotlib.pyplot as plt

from lhcsmapi.Time import Time


def get_max_diff_val_and_idx(u_nqps_diff_dfs, t_st_magnet_ok, quenched_magnets):
    # take only certain period of time
    # # a fix in case after t_st_magnet_ok QH resulted in voltage increase beyond the threshold
    u_nqps_diff_slice_dfs = []
    for u_nqps_diff_df in u_nqps_diff_dfs:
        mask = (u_nqps_diff_df.index > t_st_magnet_ok - 0.2) & (u_nqps_diff_df.index < t_st_magnet_ok)
        u_nqps_diff_slice_dfs.append(u_nqps_diff_df[mask])

    if all([u_nqps_diff_slice_df.empty for u_nqps_diff_slice_df in u_nqps_diff_slice_dfs]):
        return -math.inf, 0

    u_nqps_diff_no_quench_dfs = []
    for u_nqps_diff_slice_df in u_nqps_diff_slice_dfs:
        col = u_nqps_diff_slice_df.columns[0]
        if 'U_REF' in col or not all([quenched_magnet in col for quenched_magnet in quenched_magnets]):
            u_nqps_diff_no_quench_dfs.append(u_nqps_diff_slice_df)

    # ToDo with FeatureBuilder
    max_idx = [u_nqps_diff_no_quench_df.max(axis=1).idxmax() for u_nqps_diff_no_quench_df in
               u_nqps_diff_no_quench_dfs]
    max_val = [u_nqps_diff_no_quench_df.max(axis=0).max() for u_nqps_diff_no_quench_df in u_nqps_diff_no_quench_dfs]
    idx_max_val = max_val.index(max(max_val))
    return max_idx[idx_max_val], max_val[idx_max_val]


def analyze_qds_trigger(u_qs0_df, st_magnet_ok_df, st_nqd0_df):

    mask = (st_magnet_ok_df.index > st_magnet_ok_df.index[10]) & \
           (st_magnet_ok_df.values == False).reshape(len(st_magnet_ok_df), )
    st_qds_sub_df = st_magnet_ok_df[mask]

    if len(st_qds_sub_df) > 0:
        t_st_magnet_ok = st_qds_sub_df.index[0]

        mask = (st_nqd0_df.index > st_nqd0_df.index[10]) & \
               (st_nqd0_df.index < t_st_magnet_ok) & \
               (st_nqd0_df.values == True).reshape(len(st_nqd0_df), )

        t_st_nqd0 = st_nqd0_df[mask].index[-1]
        t_st_nqd0 = st_nqd0_df.index[st_nqd0_df.index.get_loc(t_st_nqd0) + 1]

        u_st_nqd0 = u_qs0_df[u_qs0_df.index == t_st_nqd0].values[0][0]

        if min(abs(u_qs0_df).values)[0] <= 0.01:
            u_qs0_before_quench_df = u_qs0_df[u_qs0_df.index < t_st_nqd0]

            mask = (abs(u_qs0_before_quench_df - u_qs0_before_quench_df.values[0]) <= 0.01).values.reshape(
                len(u_qs0_before_quench_df), )
            t_start_quench = u_qs0_before_quench_df[mask].index[-1]

            t_start_quench = u_qs0_df.index[u_qs0_df.index.get_loc(t_start_quench) - 1]
        else:
            t_start_quench = 0

        u_start_quench = \
            u_qs0_df.loc[u_qs0_df.index[u_qs0_df.index.get_loc(t_start_quench, method='nearest')]].values[0]

        du_dt = (u_st_nqd0 - u_start_quench) / (t_st_nqd0 - t_start_quench)

        t_delay_qh_trigger = -t_st_magnet_ok
        analysis_comment = ''
    else:
        t_st_magnet_ok = u_qs0_df.index[-1] - 0.075
        t_st_nqd0 = u_qs0_df.index[-1] - 0.075
        t_start_quench = u_qs0_df.index[0] + 0.025
        u_start_quench = math.nan
        du_dt = 0
        t_delay_qh_trigger = math.nan
        analysis_comment = 'Seems that the heaters were triggered by nQPS'

    COLUMN_NAMES = ["t_st_nqd0 [s]", "t_st_magnet_ok [s]", "u_st_nqd0 [V]", "t_start_quench [s]", "du_dt [V/s]",
                    "t_delay_qh_trigger [s]", "analysis_comment"]
    value = [t_st_nqd0, t_st_magnet_ok, u_start_quench, t_start_quench, du_dt, t_delay_qh_trigger, analysis_comment]
    return pd.DataFrame(columns=COLUMN_NAMES, data=[value])


def find_dfs_with_quenched_magnet(u_nqps_shift_diff_df, source_qds):
    u_nqps_shift_diff_filt_dfs = []
    for u_nqps_shift_diff_df_el in u_nqps_shift_diff_df:
        if source_qds in u_nqps_shift_diff_df_el.columns.values[0]:
            u_nqps_shift_diff_filt_dfs.append(u_nqps_shift_diff_df_el)
    return u_nqps_shift_diff_filt_dfs


def plot_u_diode_cals(u_nqps_dfs, timestamp_nqps, circuit_name, i_meas_df=None, ax=None, xlim=None):
    if ax is None:
        title = '{} {} U_DIODE_RB(t) - CALS'.format(Time.to_string_short(timestamp_nqps), circuit_name)
        fig, ax1 = plt.subplots(figsize=(15, 7))
    else:
        title = 'nQPS'
        ax1 = ax

    for u_nqps_df in u_nqps_dfs:
        u_nqps_df.plot(ax=ax1, title=title, grid=True)

    ax1.title.set_size(20)
    ax1.legend(loc='lower left')
    ax1.set_xlabel("time, [s]", fontsize=15)
    ax1.set_ylabel("U_DIODE_RB, [V]", fontsize=15)
    ax1.tick_params(labelsize=15)

    if xlim is not None:
        ax1.set_xlim(xlim)

    if i_meas_df is not None:
        ax2 = ax1.twinx()
        i_meas_df.plot(ax=ax2, color='C4')
        ax2.legend(loc='upper right')
        ax2.tick_params(labelsize=15)
        ax2.set_ylabel("I_MEAS, [A]", color='C4', fontsize=15)

    ax1.grid(True)

    if ax is None:
        plt.show()


def plot_u_diode_diff_pm(u_nqps_diff_dfs, timestamp_nqps, circuit_name, t_st_magnet_ok, ax=None):
    if ax is None:
        title = '{} {} U_DIODE_RB(t) - CALS'.format(Time.to_string_short(timestamp_nqps), circuit_name)
        fig, ax1 = plt.subplots(figsize=(15, 7))
    else:
        title = 'nQPS'
        ax1 = ax

    for u_nqps_diff_df in u_nqps_diff_dfs:
        u_nqps_diff_df.plot(ax=ax1, title=title, grid=True)

    ax1.axvspan(xmin=-0.2, xmax=t_st_magnet_ok, ymin=0, ymax=0.5, facecolor='xkcd:yellowgreen')
    ax1.set_ylabel("U (Calculated diode differences), [V]", fontsize=15)
    ax1.legend(loc='lower left')
    ax1.set_xlabel("time, [s]", fontsize=15)
    ax1.set_xlim(t_st_magnet_ok - 0.2, t_st_magnet_ok + 0.1)
    ax1.set_ylim(0, 2)
    ax1.tick_params(labelsize=15)


def plot_u_diode_pm_nqds(u_nqps_dfs, timestamp_nqps, circuit_name, ax=None):
    if ax is None:
        title = '{} {} U_DIODE_RB(t) and ST_MAGNET_OK_SQRB(t) - PM'.format(
            Time.to_string_short(timestamp_nqps), circuit_name)
        fig, ax1 = plt.subplots(figsize=(15, 7))
    else:
        title = 'nQPS'
        ax1 = ax

    for u_nqps_df in u_nqps_dfs:
        u_nqps_df.plot(ax=ax1, title=title, grid=True)

    ax1.title.set_size(20)
    ax1.legend(loc='lower left')
    ax1.set_xlabel("time, [s]", fontsize=15)
    ax1.set_ylabel("U_DIODE_RB, [V]", fontsize=15)
    ax1.tick_params(labelsize=15)

    if ax is None:
        plt.show()


def _get_magnet_name(signal_name):
    magnet_name = signal_name.split('.')[1].split(':')[0]
    if 'U_REF' in signal_name:
        return magnet_name + ':U_REF'
    else:
        return magnet_name


def calculate_col_diff(u_diode_ref_dfs):
    diff_dfs = []
    for index, df_first in enumerate(u_diode_ref_dfs):
        magnet_first = _get_magnet_name(df_first.columns[0])
        for df_second in u_diode_ref_dfs[index + 1:]:
            magnet_second = _get_magnet_name(df_second.columns[0])
            diff_name = '{}-{}'.format(magnet_first, magnet_second)
            value = abs(df_first.values - df_second.values)
            time = df_first.index
            diff_df = pd.DataFrame(index=time, data=value, columns=[diff_name])
            diff_dfs.append(diff_df)

    return diff_dfs


def synchronize_nqps_to_iqps(u_diode_ref_dfs, timestamp_nqps, timestamp_qds):
    u_nqps_shift_dfs = deepcopy(u_diode_ref_dfs)
    for u_nqps_shift_df in u_nqps_shift_dfs:
        if not math.isnan(timestamp_nqps):
            u_nqps_shift_df.index = u_nqps_shift_df.index + (timestamp_nqps - timestamp_qds) * 1e-9
        else:
            u_nqps_shift_df.index = u_nqps_shift_df.index + 10

    return u_nqps_shift_dfs


def plot_u_qds(u_qs0_df, u_1_df, u_2_df, timestamp_qds, timestamp_pic, *, title, xmin, xmax, xlim, ax=None):
    if ax is None:
        ax1 = u_qs0_df.plot(title=title, figsize=(15, 7.5), color='C0', grid=True, xlim=xlim, legend=True)
    else:
        ax1 = u_qs0_df.plot(ax=ax, title='iQPS', color='C0', grid=True, xlim=xlim, legend=True)

    ax1.title.set_size(20)
    ax1.axvline(x=timestamp_pic - timestamp_qds * 1e-9, color='green', linestyle='--', linewidth=1.5)
    ax1.legend(['U_QS0'], loc='lower left')
    ax1.set_ylabel("U_QS0, [V]", color='C0', fontsize=15)
    ax1.set_xlabel("time, [s]", fontsize=15)
    ax1.tick_params(labelsize=15)
    ax1.axhspan(ymin=-0.1, ymax=+0.1, facecolor='xkcd:yellowgreen')
    ax1.axvspan(xmin=xmin, xmax=xmax, facecolor='xkcd:goldenrod')

    ax2 = ax1.twinx()
    u_1_df.plot(ax=ax2, color='C1', xlim=xlim)
    u_2_df.plot(ax=ax2, color='C2', xlim=xlim)
    ax2.legend(['U_1', 'U_2'], loc='upper right')
    ax2.set_ylabel("U_1, U_2, [V]", fontsize=15)
    ax2.set_xlabel("time, [s]", fontsize=15)
    ax2.tick_params(labelsize=15)

    if ax is None:
        plt.show()


def plot_st_qds(u_qs0_df, st_magnet_ok_df, st_nqd0_df, *, title, xmin, xmax, xlim, ax=None):
    if ax is None:
        ax1 = u_qs0_df.plot(title=title, figsize=(15, 7.5), style='o-', grid=True, xlim=xlim, legend=True)
    else:
        ax1 = u_qs0_df.plot(ax=ax, style='o-', grid=True, xlim=xlim, legend=True)

    ax1.title.set_size(20)
    ax1.legend(['U_QS0'], loc='lower left')
    ax1.set_xlabel("time, [s]", fontsize=15)
    ax1.set_ylabel("U_QS0, [V]", color='C0', fontsize=15)
    ax1.tick_params(labelsize=15)
    ax1.axhspan(ymin=-0.1, ymax=+0.1, facecolor='xkcd:yellowgreen')  # beige
    ax1.axvspan(xmin=xmin, xmax=xmax, facecolor='xkcd:goldenrod')

    ax2 = ax1.twinx()
    st_magnet_ok_df.astype(int).plot(ax=ax2, yticks=[0, 1], ylim=[-0.1, 1.1], drawstyle="steps-post",
                                     style={'ST_MAGNET_OK': 'ro-'}, xlim=xlim, grid=True)
    st_nqd0_df.astype(int).plot(ax=ax2, yticks=[0, 1], ylim=[-0.1, 1.1], style={'ST_NQD0': 'bx-'},
                                drawstyle="steps-post", xlim=xlim, grid=True)

    ax2.set_ylabel("ST_NDQ0, ST_MAGNET_OK, [-]", fontsize=15)
    ax2.legend(['ST_NDQ0', 'ST_MAGNET_OK'], loc='upper right')
    ax2.tick_params(labelsize=15)

    if ax is None:
        plt.show()