import getpass
import functools

import pandas as pd
from IPython.display import display
from ipywidgets import Dropdown, Text, HBox, VBox, Select, Label, ToggleButtons, Layout

from lhcsmapi.gui.pc.FgcPmSearchModuleMediator import get_circuit_names_or_types
from lhcsmapi.gui.ModuleMediator import ModuleMediator
from lhcsmapi.metadata.SignalMetadata import SignalMetadata


class HwcSearchModuleMediator(ModuleMediator):

    def __init__(self, circuit_type, hwc_test, hwc_summary_path):
        self._circuit_type = circuit_type
        self.hwc_test = hwc_test
        self.hwc_summary = pd.read_csv(hwc_summary_path)[
            ['systemName', 'campaign', 'testName', 'executionStartTime', 'executionEndTime', 'executedTestStatus']]
        self.hwc_summary_circuit = self.hwc_summary
        # First row
        circuit_names = get_circuit_names_or_types(
            circuit_type) if circuit_type != 'RQ' else SignalMetadata.get_circuit_names('RQ')
        description = 'Circuit type:' if self._circuit_type == '600A' else 'Circuit name'
        self.select_circuit_name_dropdown = Dropdown(
            options=circuit_names,
            description=description,
            disabled=False,
        )

        self.select_circuit_name_dropdown.observe(
            functools.partial(HwcSearchModuleMediator.update_hwc_table, self=self),
            names=['value'])

        first_row = HBox(children=[self.select_circuit_name_dropdown])

        # Second row
        self.done_by_txt = Text(placeholder='Your name', description='Done by:', value=getpass.getuser(),
                                disabled=True)

        self.analysis_mode_tgl_btn = ToggleButtons(
            options=['Automatic', 'Manual'],
            description='Analysis:',
            disabled=False,
            button_style='',
            tooltips=['Select automatic for automatic execution, comment, and acceptance.',
                      'Select manual to execute, comment, and accept each analysis manually.']
        )
        second_row = HBox(children=[self.done_by_txt, self.analysis_mode_tgl_btn])

        # Third row
        self.hwc_sel = Select(
            options=[],
            rows=15,
            disabled=False,
            layout=Layout(width='90%', font='monospace')
        )
        HwcSearchModuleMediator.update_hwc_table(None, self)

        third_row = HBox(children=[Label(value='HWC tests:'), self.hwc_sel])

        # Final GUI
        self.widget = VBox(children=[first_row, second_row, third_row])

        display(self.widget)

    @staticmethod
    def update_hwc_table(value, self):
        self.hwc_summary_circuit = self.hwc_summary[
            (self.hwc_summary['systemName'] == self.select_circuit_name_dropdown.value) & (
                self.hwc_summary['testName'].str.contains(self.hwc_test)) & (
                        self.hwc_summary['executedTestStatus'] != 'RUNNING')]
        self.hwc_summary_circuit.reset_index(drop=True, inplace=True)

        # find max string length
        max_str_lens = []
        for col in self.hwc_summary_circuit.columns:
            max_str_lens.append(len(max(self.hwc_summary_circuit[col].values, key=len)))

        options = []
        for index, row in self.hwc_summary_circuit.iterrows():
            aligned_values = [(max_str_len - len(value)) * '  ' + value for max_str_len, value in
                              zip(max_str_lens, row.values)]
            options.append(' | '.join(aligned_values))

        self.hwc_sel.options = options

    def get_start_time(self):
        return self.hwc_summary_circuit.loc[self.hwc_sel.index, 'executionStartTime']

    def get_end_time(self):
        return self.hwc_summary_circuit.loc[self.hwc_sel.index, 'executionEndTime']