import unittest
import warnings
from unittest.mock import patch

import numpy as np
import pandas as pd

from lhcsmapi.analysis.IpdCircuitAnalysis import IpdCircuitAnalysis
from test.resources.read_csv import read_csv


class TestIpdCircuitAnalysis(unittest.TestCase):

    _circuit_type = 'IPD2_B1B2'
    _is_automatic = True
    _circuit_name = 'RD3.R4'
    _timestamp_qds = 1543842413329000000

    _results_table = pd.DataFrame({'Circuit Name': {0: 'RD3.R4'}, 'Date': {0: '2018-12-03'},
                                   'Time': {0: '14:06:53.340'}, 'I_MEAS': {0: 5818.8}, 'MIITs_DCCT': {0: 5.672},
                                   'Ramp Rate': {0: 18.1}, 'Plateau Duration': {0: np.nan}, 'max_I_EARTH': {0: -0.038},
                                   'Reason': {0: np.nan}, 'Quench Origin': {0: np.nan},
                                   'dUres/dt': {0: -111.5722660000001}, 'QH': {0: 'OK'}, 'performed_by': {0: ''},
                                   'lhcsmapi_version': {0: '1.3.257'}, 'timestamp_fgc': {0: 1543842413340000000},
                                   'timestamp_pic': {0: 1543842413329000000},
                                   'timestamp_qds_a': {0: 1543842413329000000},
                                   'timestamp_qds_b': {0: 1543842413330000000},
                                   'RD3.R4_LEADS_U_HTS': {0: 'U_HTS < quench threshold'},
                                   'RD3.R4_LEADS_U_RES': {0: 'U_RES < quench threshold'}})

    @patch("lhcsmapi.pyedsl.PlotBuilder.plt.show")
    def test_analyze_qh(self, mock_show=None):
        # arrange
        u_hds_dfs = [read_csv('resources/hwc/ipd', 'U_HDS_1_B1'), read_csv('resources/hwc/ipd', 'U_HDS_1_B2')]

        u_hds_ref_dfs = [read_csv('resources/hwc/ipd', 'U_HDS_REF_1_B1'),
                         read_csv('resources/hwc/ipd', 'U_HDS_REF_1_B2')]

        # act
        analysis = IpdCircuitAnalysis(self._circuit_type, self._results_table, is_automatic=self._is_automatic)
        analysis.analyze_qh(self._circuit_name, self._timestamp_qds, u_hds_dfs, u_hds_ref_dfs)

        # assert
        self.assertEqual('OK', analysis.results_table.loc[0, 'QH'])
        if mock_show is not None:
            mock_show.assert_called()

    def test_create_mp3_results_table(self):
        # arrange

        # act
        analysis = IpdCircuitAnalysis(self._circuit_type, self._results_table, is_automatic=self._is_automatic)
        mp3_results_table_act = analysis.create_mp3_results_table()

        # assert
        mp3_results_table_ref = pd.DataFrame({'Circuit Name': {0: 'RD3.R4'}, 'Date': {0: '2018-12-03'},
                                              'Time': {0: '14:06:53.340'}, 'flagged in APEX': {0: np.nan},
                                              'Quench Current': {0: 5818.8}, 'DCCT MIITs': {0: 5.672},
                                              'Reason': {0: np.nan}, 'Quench Origin': {0: np.nan},
                                              'Ramp Rate': {0: 18.1}, 'Plateau Duration': {0: np.nan},
                                              'dt_thres': {0: -0.0008962800845149091}, 'Comment': {0: np.nan},
                                              'Person who analyzed': {0: ''}, 'Reference Doc': {0: np.nan}})
        mp3_results_table_act.fillna(np.nan, inplace=True)
        mp3_results_table_ref.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(mp3_results_table_ref, mp3_results_table_act)


